@@ -12,7 +12,6 @@ import os as _os
 import platform as _platform
 import signal as _signal
 import sys as _sys
-import types as _types
 import warnings as _warnings
 
 import docarray as _docarray
@@ -60,7 +59,7 @@ elif _sys.version_info >= (3, 8, 0) and _platform.system() == 'Darwin':
 # this is managed by git tag and updated on every release
 # NOTE: this represents the NEXT release version
 
-__version__ = '3.2.7'
+__version__ = '3.3.4'
 
 # do not change this line manually
 # this is managed by proto/build-proto.sh and updated on every execution
@@ -104,6 +103,7 @@ __jina_env__ = (
 __default_host__ = _os.environ.get(
     'JINA_DEFAULT_HOST', '127.0.0.1' if __windows__ else '0.0.0.0'
 )
+__default_port_monitoring__ = 9090
 __docker_host__ = 'host.docker.internal'
 __default_executor__ = 'BaseExecutor'
 __default_reducer_executor__ = 'ReducerExecutor'
