@@ -16,17 +16,75 @@ from pathlib import Path
 from typing import Tuple, Optional, Dict
 from urllib.parse import urlparse, urljoin
 from urllib.request import Request, urlopen
+from contextlib import nullcontext
 
-from .. import __resources_path__
-from ..importer import ImportExtensions
-from ..logging.predefined import default_logger
+from jina import __resources_path__
+from jina.importer import ImportExtensions
+from jina.logging.predefined import default_logger
 
 
 @lru_cache()
-def get_hubble_url() -> str:
-    """Get the Hubble URL from api.jina.ai or os.environ
+def _get_hub_root() -> Path:
+    hub_root = Path(os.environ.get('JINA_HUB_ROOT', Path.home().joinpath('.jina')))
 
-    :return: the Hubble URL
+    if not hub_root.exists():
+        hub_root.mkdir(parents=True, exist_ok=True)
+
+    return hub_root
+
+
+@lru_cache()
+def get_hub_packages_dir() -> Path:
+    """Get the path of folder where the hub packages are stored
+
+    :return: the path of folder where the hub packages are stored
+    """
+    root = _get_hub_root()
+    hub_packages = root.joinpath('hub-package')
+
+    if not hub_packages.exists():
+        hub_packages.mkdir(parents=True, exist_ok=True)
+
+    return hub_packages
+
+
+@lru_cache()
+def get_cache_db() -> Path:
+    """Get the path of cache db of hub Executors
+
+    :return: the path of cache db of hub Executors
+    """
+    root = _get_hub_root()
+    cache_db = root.joinpath('disk_cache.db')
+
+    return cache_db
+
+
+@lru_cache()
+def get_download_cache_dir() -> Path:
+    """Get the path of cache folder where the downloading cache is stored
+
+    :return: the path of cache folder where the downloading cache is stored
+    """
+    root = _get_hub_root()
+    cache_dir = Path(
+        os.environ.get(
+            'JINA_HUB_CACHE_DIR',
+            root.joinpath('.cache'),
+        )
+    )
+
+    if not cache_dir.exists():
+        cache_dir.mkdir(parents=True, exist_ok=True)
+
+    return cache_dir
+
+
+@lru_cache()
+def _get_hubble_base_url() -> str:
+    """Get base Hubble Url from api.jina.ai or os.environ
+
+    :return: base Hubble Url
     """
     if 'JINA_HUBBLE_REGISTRY' in os.environ:
         u = os.environ['JINA_HUBBLE_REGISTRY']
@@ -40,10 +98,29 @@ def get_hubble_url() -> str:
                 u = json.load(resp)['url']
         except:
             default_logger.critical(
-                'Can not fetch the URL of Hubble from `api.jina.ai`'
+                'Can not fetch the Url of Hubble from `api.jina.ai`'
             )
             raise
-    return urljoin(u, '/v1/executors')
+
+    return u
+
+
+def get_hubble_url_v1() -> str:
+    """Get v1 Hubble Url
+
+    :return: v1 Hubble url
+    """
+    u = _get_hubble_base_url()
+    return urljoin(u, '/v1')
+
+
+def get_hubble_url_v2() -> str:
+    """Get v2 Hubble Url
+
+    :return: v2 Hubble url
+    """
+    u = _get_hubble_base_url()
+    return urljoin(u, '/v2')
 
 
 def parse_hub_uri(uri_path: str) -> Tuple[str, str, str, str]:
@@ -54,7 +131,7 @@ def parse_hub_uri(uri_path: str) -> Tuple[str, str, str, str]:
     """
     parser = urlparse(uri_path)
     scheme = parser.scheme
-    if scheme not in {'jinahub', 'jinahub+docker'}:
+    if scheme not in {'jinahub', 'jinahub+docker', 'jinahub+sandbox'}:
         raise ValueError(f'{uri_path} is not a valid Hub URI.')
 
     items = list(parser.netloc.split(':'))
@@ -145,7 +222,7 @@ def archive_package(package_folder: 'Path') -> 'io.BytesIO':
 
         for p in path.iterdir():
             rel_path = p.relative_to(base_path)
-            if ignored_spec.match_file(rel_path):
+            if ignored_spec.match_file(str(rel_path)):
                 continue
             if p.is_dir():
                 _zip(base_path, p, archive)
@@ -217,7 +294,12 @@ def download_with_resume(
     _download(url, filepath, _resume_byte_pos)
 
     if md5sum and not md5file(filepath) == md5sum:
-        raise RuntimeError('MD5 checksum failed.')
+        raise RuntimeError(
+            'MD5 checksum failed.'
+            'Might happen when the network is unstable, please retry.'
+            'If still not work, feel free to raise an issue.'
+            'https://github.com/jina-ai/jina/issues/new'
+        )
 
     return filepath
 
@@ -277,38 +359,54 @@ def disk_cache_offline(
     def decorator(func):
         @wraps(func)
         def wrapper(*args, **kwargs):
+
             call_hash = f'{func.__name__}({", ".join(map(str, args))})'
 
             pickle_protocol = 4
+            file_lock = nullcontext()
+            with ImportExtensions(
+                required=False,
+                help_text=f'FileLock is needed to guarantee non-concurrent access to the'
+                f'cache_file {cache_file}',
+            ):
+                import filelock
 
-            try:
-                cache_db = shelve.open(
-                    cache_file, protocol=pickle_protocol, writeback=True
-                )
-            except Exception:
-                if os.path.exists(cache_file):
-                    # cache is in an unsupported format, reset the cache
-                    os.remove(cache_file)
+                file_lock = filelock.FileLock(f'{cache_file}.lock', timeout=-1)
+
+            cache_db = None
+            with file_lock:
+                try:
                     cache_db = shelve.open(
                         cache_file, protocol=pickle_protocol, writeback=True
                     )
-                else:
-                    raise
+                except Exception:
+                    if os.path.exists(cache_file):
+                        # cache is in an unsupported format, reset the cache
+                        os.remove(cache_file)
+                        cache_db = shelve.open(
+                            cache_file, protocol=pickle_protocol, writeback=True
+                        )
 
-            with cache_db as dict_db:
-                try:
-                    if call_hash in dict_db and not kwargs.get('force', False):
-                        return dict_db[call_hash]
+            if cache_db is None:
+                # if we failed to load cache, do not raise, it is only an optimization thing
+                return func(*args, **kwargs), False
+            else:
+                with cache_db as dict_db:
+                    try:
+                        if call_hash in dict_db and not kwargs.get('force', False):
+                            return dict_db[call_hash], True
 
-                    result = func(*args, **kwargs)
-                    dict_db[call_hash] = result
-                except urllib.error.URLError:
-                    if call_hash in dict_db:
-                        default_logger.warning(message.format(func_name=func.__name__))
-                        return dict_db[call_hash]
-                    else:
-                        raise
-            return result
+                        result = func(*args, **kwargs)
+                        dict_db[call_hash] = result
+                    except urllib.error.URLError:
+                        if call_hash in dict_db:
+                            default_logger.warning(
+                                message.format(func_name=func.__name__)
+                            )
+                            return dict_db[call_hash], True
+                        else:
+                            raise
+                return result, False
 
         return wrapper
 
