@@ -27,7 +27,7 @@ from typing import (
 from rich import print
 from rich.table import Table
 
-from jina import __default_host__, helper
+from jina import __default_host__, __default_port_monitoring__, __docker_host__, helper
 from jina.clients import Client
 from jina.clients.mixin import AsyncPostMixin, PostMixin
 from jina.enums import (
@@ -39,6 +39,7 @@ from jina.enums import (
 from jina.excepts import (
     FlowMissingDeploymentError,
     FlowTopologyError,
+    PortAlreadyUsed,
     RuntimeFailToStart,
 )
 from jina.helper import (
@@ -51,6 +52,7 @@ from jina.helper import (
     get_internal_ip,
     get_public_ip,
     get_rich_console,
+    is_port_free,
     typename,
 )
 from jina.jaml import JAMLCompatible
@@ -66,6 +68,8 @@ from jina.parsers.flow import set_flow_parser
 
 __all__ = ['Flow']
 
+from jina.serve.networking import host_is_local, in_docker
+
 
 class FlowType(type(ExitStack), type(JAMLCompatible)):
     """Type of Flow, metaclass of :class:`BaseFlow`"""
@@ -114,7 +118,7 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         :param protocol: Communication protocol between server and client.
         :param proxy: If set, respect the http_proxy and https_proxy environment variables. otherwise, it will unset these proxy variables before start. gRPC seems to prefer no proxy
         :param return_responses: If set, return results as List of Requests instead of a reduced DocArray.
-        :param tls: If set, connect to gateway using https
+        :param tls: If set, connect to gateway using tls encryption
 
         .. # noqa: DAR202
         .. # noqa: DAR101
@@ -133,6 +137,7 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         cors: Optional[bool] = False,
         default_swagger_ui: Optional[bool] = False,
         deployments_addresses: Optional[str] = '{}',
+        deployments_disable_reduce: Optional[str] = '[]',
         description: Optional[str] = None,
         disable_reduce: Optional[bool] = False,
         env: Optional[dict] = None,
@@ -140,9 +145,11 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         expose_graphql_endpoint: Optional[bool] = False,
         graph_conditions: Optional[str] = '{}',
         graph_description: Optional[str] = '{}',
+        grpc_server_kwargs: Optional[dict] = None,
         host: Optional[str] = '0.0.0.0',
         host_in: Optional[str] = '0.0.0.0',
         log_config: Optional[str] = None,
+        monitoring: Optional[bool] = False,
         name: Optional[str] = 'gateway',
         native: Optional[bool] = False,
         no_crud_endpoints: Optional[bool] = False,
@@ -150,6 +157,7 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         output_array_type: Optional[str] = None,
         polling: Optional[str] = 'ANY',
         port: Optional[int] = None,
+        port_monitoring: Optional[int] = 9090,
         prefetch: Optional[int] = 0,
         protocol: Optional[str] = 'GRPC',
         proxy: Optional[bool] = False,
@@ -157,9 +165,10 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         quiet: Optional[bool] = False,
         quiet_error: Optional[bool] = False,
         replicas: Optional[int] = 1,
-        runtime_backend: Optional[str] = 'PROCESS',
         runtime_cls: Optional[str] = 'GRPCGatewayRuntime',
         shards: Optional[int] = 1,
+        ssl_certfile: Optional[str] = None,
+        ssl_keyfile: Optional[str] = None,
         timeout_ctrl: Optional[int] = 60,
         timeout_ready: Optional[int] = 600000,
         title: Optional[str] = None,
@@ -180,6 +189,7 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         :param cors: If set, a CORS middleware is added to FastAPI frontend to allow cross-origin access.
         :param default_swagger_ui: If set, the default swagger ui is used for `/docs` endpoint.
         :param deployments_addresses: dictionary JSON with the input addresses of each Deployment
+        :param deployments_disable_reduce: list JSON disabling the built-in merging mechanism for each Deployment listed
         :param description: The description of this HTTP server. It will be used in automatics docs such as Swagger UI.
         :param disable_reduce: Disable the built-in reduce mechanism, set this if the reduction is to be handled by the Executor connected to this Head
         :param env: The map of environment variables that are available inside runtime
@@ -187,9 +197,11 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         :param expose_graphql_endpoint: If set, /graphql endpoint is added to HTTP interface.
         :param graph_conditions: Dictionary stating which filtering conditions each Executor in the graph requires to receive Documents.
         :param graph_description: Routing graph for the gateway
+        :param grpc_server_kwargs: Dictionary of kwargs arguments that will be passed to the grpc server when starting the server # todo update
         :param host: The host address of the runtime, by default it is 0.0.0.0.
         :param host_in: The host address for binding to, by default it is 0.0.0.0
         :param log_config: The YAML config of the logger used in this object.
+        :param monitoring: If set, spawn an http server with a prometheus endpoint to expose metrics
         :param name: The name of this object.
 
           This will be used in the following places:
@@ -218,6 +230,7 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
               JSON dict, {endpoint: PollingType}
               {'/custom': 'ALL', '/search': 'ANY', '*': 'ANY'}
         :param port: The port for input data to bind to, default is a random port between [49152, 65535]
+        :param port_monitoring: The port on which the prometheus server is exposed, default port is 9090
         :param prefetch: Number of requests fetched from the client before feeding into the first Executor.
 
               Used to control the speed of data input into a Flow. 0 disables prefetch (disabled by default)
@@ -232,9 +245,10 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         :param quiet: If set, then no log will be emitted from this object.
         :param quiet_error: If set, then exception stack information will not be added to the log
         :param replicas: The number of replicas in the deployment
-        :param runtime_backend: The parallel backend of the runtime inside the Pod
         :param runtime_cls: The runtime class to run inside the Pod
         :param shards: The number of shards in the deployment running at the same time. For more details check https://docs.jina.ai/fundamentals/flow/create-flow/#complex-flow-topologies
+        :param ssl_certfile: the path to the certificate file
+        :param ssl_keyfile: the path to the key file
         :param timeout_ctrl: The timeout in milliseconds of the control request, -1 for waiting forever
         :param timeout_ready: The timeout in milliseconds of a Pod waits for the runtime to be ready, -1 for waiting forever
         :param title: The title of this HTTP server. It will be used in automatics docs such as Swagger UI.
@@ -444,6 +458,7 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         graph_description: Dict[str, List[str]],
         deployments_addresses: Dict[str, List[str]],
         graph_conditions: Dict[str, Dict],
+        deployments_disabled_reduce: List[str],
         **kwargs,
     ):
         kwargs.update(
@@ -460,6 +475,11 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
 
         kwargs.update(self._common_kwargs)
         args = ArgNamespace.kwargs2namespace(kwargs, set_gateway_parser())
+
+        # We need to check later if the port was manually set or randomly
+        args.default_port = (
+            kwargs.get('port', None) is None and kwargs.get('port_expose', None) is None
+        )
         args.noblock_on_start = True
         args.expose_graphql_endpoint = (
             self.args.expose_graphql_endpoint
@@ -467,6 +487,7 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         args.graph_description = json.dumps(graph_description)
         args.graph_conditions = json.dumps(graph_conditions)
         args.deployments_addresses = json.dumps(deployments_addresses)
+        args.deployments_disable_reduce = json.dumps(deployments_disabled_reduce)
         self._deployment_nodes[GATEWAY_NAME] = Deployment(args, needs)
 
     def _get_deployments_addresses(self) -> Dict[str, List[str]]:
@@ -474,7 +495,15 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         for node, v in self._deployment_nodes.items():
             if node == 'gateway':
                 continue
-            graph_dict[node] = [f'{v.protocol}://{v.host}:{v.head_port}']
+            if v.head_args:
+                # add head information
+                graph_dict[node] = [f'{v.protocol}://{v.host}:{v.head_port}']
+            else:
+                # there is no head, add the worker connection information instead
+                host = v.host
+                if host_is_local(host) and in_docker() and v.dockerized_uses:
+                    host = __docker_host__
+                graph_dict[node] = [f'{v.protocol}://{host}:{port}' for port in v.ports]
 
         return graph_dict
 
@@ -491,13 +520,18 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
 
             if v.external:
                 deployment_k8s_address = f'{v.host}'
-            else:
+            elif v.head_args:
                 deployment_k8s_address = (
                     f'{to_compatible_name(v.head_args.name)}.{k8s_namespace}.svc'
                 )
+            else:
+                deployment_k8s_address = (
+                    f'{to_compatible_name(v.name)}.{k8s_namespace}.svc'
+                )
 
+            external_port = v.head_port if v.head_port else v.port
             graph_dict[node] = [
-                f'{deployment_k8s_address}:{v.head_port if v.external else GrpcConnectionPool.K8S_PORT}'
+                f'{deployment_k8s_address}:{external_port if v.external else GrpcConnectionPool.K8S_PORT}'
             ]
 
         return graph_dict if graph_dict else None
@@ -510,9 +544,14 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         for node, v in self._deployment_nodes.items():
             if node == 'gateway':
                 continue
-            deployment_docker_compose_address = (
-                f'{to_compatible_name(v.head_args.name)}:{port}'
-            )
+            elif v.head_args:
+                deployment_docker_compose_address = (
+                    f'{to_compatible_name(v.head_args.name)}:{port}'
+                )
+            else:
+                deployment_docker_compose_address = (
+                    f'{to_compatible_name(v.name)}:{port}'
+                )
             graph_dict[node] = [deployment_docker_compose_address]
 
         return graph_dict
@@ -525,6 +564,14 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
 
         return graph_condition
 
+    def _get_disabled_reduce_deployments(self) -> List[str]:
+        disabled_deployments = []
+        for node, v in self._deployment_nodes.items():
+            if v.args.disable_reduce:
+                disabled_deployments.append(node)
+
+        return disabled_deployments
+
     def _get_graph_representation(self) -> Dict[str, List[str]]:
         def _add_node(graph, n):
             # in the graph we need to distinguish between start and end gateway, although they are the same deployment
@@ -599,6 +646,7 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         *,
         compression: Optional[str] = 'NoCompression',
         connection_list: Optional[str] = None,
+        disable_auto_volume: Optional[bool] = False,
         disable_reduce: Optional[bool] = False,
         docker_kwargs: Optional[dict] = None,
         entrypoint: Optional[str] = None,
@@ -610,19 +658,19 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         host_in: Optional[str] = '0.0.0.0',
         install_requirements: Optional[bool] = False,
         log_config: Optional[str] = None,
+        monitoring: Optional[bool] = False,
         name: Optional[str] = None,
         native: Optional[bool] = False,
         output_array_type: Optional[str] = None,
         polling: Optional[str] = 'ANY',
         port: Optional[int] = None,
-        port_jinad: Optional[int] = 8000,
+        port_monitoring: Optional[int] = 9090,
         pull_latest: Optional[bool] = False,
         py_modules: Optional[List[str]] = None,
         quiet: Optional[bool] = False,
         quiet_error: Optional[bool] = False,
         quiet_remote_logs: Optional[bool] = False,
         replicas: Optional[int] = 1,
-        runtime_backend: Optional[str] = 'PROCESS',
         runtime_cls: Optional[str] = 'WorkerRuntime',
         shards: Optional[int] = 1,
         timeout_ctrl: Optional[int] = 60,
@@ -645,6 +693,7 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
 
         :param compression: The compression mechanism used when sending requests from the Head to the WorkerRuntimes. Possibilities are `NoCompression, Gzip, Deflate`. For more details, check https://grpc.github.io/grpc/python/grpc.html#compression.
         :param connection_list: dictionary JSON with a list of connections to configure
+        :param disable_auto_volume: Do not automatically mount a volume for dockerized Executors.
         :param disable_reduce: Disable the built-in reduce mechanism, set this if the reduction is to be handled by the Executor connected to this Head
         :param docker_kwargs: Dictionary of kwargs arguments that will be passed to Docker SDK when starting the docker '
           container.
@@ -666,6 +715,7 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         :param host_in: The host address for binding to, by default it is 0.0.0.0
         :param install_requirements: If set, install `requirements.txt` in the Hub Executor bundle to local
         :param log_config: The YAML config of the logger used in this object.
+        :param monitoring: If set, spawn an http server with a prometheus endpoint to expose metrics
         :param name: The name of this object.
 
           This will be used in the following places:
@@ -690,7 +740,7 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
               JSON dict, {endpoint: PollingType}
               {'/custom': 'ALL', '/search': 'ANY', '*': 'ANY'}
         :param port: The port for input data to bind to, default is a random port between [49152, 65535]
-        :param port_jinad: The port of the remote machine for usage with JinaD.
+        :param port_monitoring: The port on which the prometheus server is exposed, default port is 9090
         :param pull_latest: Pull the latest image before running
         :param py_modules: The customized python modules need to be imported before loading the executor
 
@@ -702,7 +752,6 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         :param quiet_error: If set, then exception stack information will not be added to the log
         :param quiet_remote_logs: Do not display the streaming of remote logs on local console
         :param replicas: The number of replicas in the deployment
-        :param runtime_backend: The parallel backend of the runtime inside the Pod
         :param runtime_cls: The runtime class to run inside the Pod
         :param shards: The number of shards in the deployment running at the same time. For more details check https://docs.jina.ai/fundamentals/flow/create-flow/#complex-flow-topologies
         :param timeout_ctrl: The timeout in milliseconds of the control request, -1 for waiting forever
@@ -726,9 +775,9 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
                   When use it under Python, one can use the following values additionally:
                   - a Python dict that represents the config
                   - a text file stream has `.read()` interface
-        :param uses_after: The executor attached after the Pods described by --uses, typically used for receiving from all shards, accepted type follows `--uses`
+        :param uses_after: The executor attached after the Pods described by --uses, typically used for receiving from all shards, accepted type follows `--uses`. This argument only applies for sharded Deployments (shards > 1).
         :param uses_after_address: The address of the uses-before runtime
-        :param uses_before: The executor attached after the Pods described by --uses, typically before sending to all shards, accepted type follows `--uses`
+        :param uses_before: The executor attached before the Pods described by --uses, typically before sending to all shards, accepted type follows `--uses`. This argument only applies for sharded Deployments (shards > 1).
         :param uses_before_address: The address of the uses-before runtime
         :param uses_metas: Dictionary of keyword arguments that will override the `metas` configuration in `uses`
         :param uses_requests: Dictionary of keyword arguments that will override the `requests` configuration in `uses`
@@ -802,17 +851,6 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
             if key not in kwargs and key != 'port':
                 kwargs[key] = value
 
-        # check if host is set to remote:port
-        if 'host' in kwargs:
-            m = re.match(_regex_port, kwargs['host'])
-            if (
-                kwargs.get('host', __default_host__) != __default_host__
-                and m
-                and 'port_jinad' not in kwargs
-            ):
-                kwargs['port_jinad'] = m.group(2)
-                kwargs['host'] = m.group(1)
-
         # update kwargs of this Deployment
         kwargs.update(
             dict(
@@ -843,6 +881,11 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
             port = helper.random_port()
             args.port = port
 
+        if len(needs) > 1 and args.external and args.disable_reduce:
+            raise ValueError(
+                'External Executors with multiple needs have to do auto reduce.'
+            )
+
         op_flow._deployment_nodes[deployment_name] = Deployment(args, needs)
 
         op_flow.last_deployment = deployment_name
@@ -1002,6 +1045,7 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
                 graph_description=op_flow._get_graph_representation(),
                 deployments_addresses=op_flow._get_deployments_addresses(),
                 graph_conditions=op_flow._get_graph_conditions(),
+                deployments_disabled_reduce=op_flow._get_disabled_reduce_deployments(),
             )
 
         removed_deployments = []
@@ -1107,9 +1151,17 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
 
         :return: this instance
         """
+
         if self._build_level.value < FlowBuildLevel.GRAPH.value:
             self.build(copy_flow=False)
 
+        port_gateway = self._deployment_nodes[GATEWAY_NAME].args.port
+
+        if not (
+            is_port_free(__default_host__, port_gateway)
+        ):  # we check if the port is not used at parsing time as well for robustness
+            raise PortAlreadyUsed(f'port:{port_gateway}')
+
         # set env only before the Deployment get started
         if self.args.env:
             for k, v in self.args.env.items():
@@ -1289,7 +1341,8 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         # plot subgraphs
         for node, v in self._deployment_nodes.items():
             deployment_nodes.append(v.name)
-            mermaid_graph.extend(v._mermaid_str)
+            deployment_mermaid = v._mermaid_str
+            mermaid_graph.extend(deployment_mermaid)
 
         for node, v in self._deployment_nodes.items():
             for need in sorted(v.needs):
@@ -1369,9 +1422,14 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
 
         # deepcopy causes the below error while reusing a Flow in Jupyter
         # 'Pickling an AuthenticationString object is disallowed for security reasons'
-        op_flow = copy.deepcopy(self) if copy_flow else self
+        # no need to deep copy if the Graph is built because no change will be made to the Flow
+        op_flow = (
+            copy.deepcopy(self)
+            if (copy_flow and self._build_level.value == FlowBuildLevel.EMPTY)
+            else self
+        )
 
-        if build:
+        if build and op_flow._build_level.value == FlowBuildLevel.EMPTY:
             op_flow.build(False)
 
         mermaid_str = op_flow._mermaid_str
@@ -1475,6 +1533,28 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
             self.enter_context(self[GATEWAY_NAME])
             self[GATEWAY_NAME].wait_start_success()
 
+    @property
+    def monitoring(self) -> bool:
+        """Return if the monitoring is enabled
+        .. # noqa: DAR201
+        """
+        if GATEWAY_NAME in self._deployment_nodes:
+            return self[GATEWAY_NAME].args.monitoring
+        else:
+            return False
+
+    @property
+    def port_monitoring(self) -> int:
+        """Return if the monitoring is enabled
+        .. # noqa: DAR201
+        """
+        if GATEWAY_NAME in self._deployment_nodes:
+            return self[GATEWAY_NAME].args.port_monitoring
+        else:
+            return self._common_kwargs.get(
+                'port_monitoring', __default_port_monitoring__
+            )
+
     @property
     def address_private(self) -> str:
         """Return the private IP address of the gateway for connecting from other machine in the same network
@@ -1541,6 +1621,12 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
                     'GraphQL UI',
                     f'[underline][cyan]http://localhost:{self.port}/graphql[/underline][/cyan]',
                 )
+        if self.monitoring:
+            address_table.add_row(
+                '📉️',
+                'Prometheus',
+                f'[underline][cyan]http://localhost:{self.port_monitoring}[/underline][/cyan]',
+            )
 
         return address_table
 
@@ -1739,30 +1825,21 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
     # for backward support
     join = needs
 
-    def rolling_update(
+    def to_k8s_yaml(
         self,
-        deployment_name: str,
-        uses_with: Optional[Dict] = None,
+        output_base_path: str,
+        k8s_namespace: Optional[str] = None,
+        include_gateway: bool = True,
     ):
         """
-        Reload all replicas of a deployment sequentially
+        Converts the Flow into a set of yaml deployments to deploy in Kubernetes.
 
-        :param deployment_name: deployment to update
-        :param uses_with: a Dictionary of arguments to restart the executor with
-        """
-        from jina.helper import run_async
+        If you don't want to rebuild image on Jina Hub,
+        you can set `JINA_HUB_NO_IMAGE_REBUILD` environment variable.
 
-        run_async(
-            self._deployment_nodes[deployment_name].rolling_update,
-            uses_with=uses_with,
-            any_event_loop=True,
-        )
-
-    def to_k8s_yaml(self, output_base_path: str, k8s_namespace: Optional[str] = None):
-        """
-        Converts the Flow into a set of yaml deployments to deploy in Kubernetes
         :param output_base_path: The base path where to dump all the yaml files
         :param k8s_namespace: The name of the k8s namespace to set for the configurations. If None, the name of the Flow will be used.
+        :param include_gateway: Defines if the gateway deployment should be included, defaults to True
         """
         import yaml
 
@@ -1774,8 +1851,15 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         k8s_namespace = k8s_namespace or self.args.name or 'default'
 
         for node, v in self._deployment_nodes.items():
-            if v.external:
+            if v.external or (node == 'gateway' and not include_gateway):
                 continue
+            if node == 'gateway' and v.args.default_port:
+                from jina.serve.networking import GrpcConnectionPool
+
+                v.args.port = GrpcConnectionPool.K8S_PORT
+                v.first_pod_args.port = GrpcConnectionPool.K8S_PORT
+                v.args.default_port = False
+
             deployment_base = os.path.join(output_base_path, node)
             k8s_deployment = K8sDeploymentConfig(
                 args=v.args,
@@ -1801,12 +1885,16 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         )
 
     def to_docker_compose_yaml(
-        self, output_path: Optional[str] = None, network_name: Optional[str] = None
+        self,
+        output_path: Optional[str] = None,
+        network_name: Optional[str] = None,
+        include_gateway: bool = True,
     ):
         """
         Converts the Flow into a yaml file to run with `docker-compose up`
         :param output_path: The output path for the yaml file
         :param network_name: The name of the network that will be used by the deployment name
+        :param include_gateway: Defines if the gateway deployment should be included, defaults to True
         """
         import yaml
 
@@ -1828,6 +1916,9 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
         services = {}
 
         for node, v in self._deployment_nodes.items():
+            if v.external or (node == 'gateway' and not include_gateway):
+                continue
+
             docker_compose_deployment = DockerComposeConfig(
                 args=v.args,
                 deployments_addresses=self._get_docker_compose_deployments_addresses(),
@@ -1851,28 +1942,6 @@ class Flow(PostMixin, JAMLCompatible, ExitStack, metaclass=FlowType):
             f'Docker compose file has been created under {output_path}. You can use it by running `{command}`'
         )
 
-    def scale(
-        self,
-        deployment_name: str,
-        replicas: int,
-    ):
-        """
-        Scale the amount of replicas of a given Executor.
-
-        :param deployment_name: deployment to update
-        :param replicas: The number of replicas to scale to
-        """
-
-        # TODO when replicas-host is ready, needs to be passed here
-
-        from jina.helper import run_async
-
-        run_async(
-            self._deployment_nodes[deployment_name].scale,
-            replicas=replicas,
-            any_event_loop=True,
-        )
-
     @property
     def client_args(self) -> argparse.Namespace:
         """Get Client settings.
